/* Box counting dimension. */

#include <stdio.h>
#include <math.h>

char *Progname;
char Usage[] = "usage: %s [file]\n";
#define USAGE() fprintf(stderr, Usage, Progname)

void doit(FILE *);

main(int argc, char **argv)
{
	register char *curr_arg;
	register int c;
	FILE *infile;

	Progname = *argv;

	/* Parse arguments beginning with a hyphen. */

	while (--argc > 0 && **++argv == '-' && *(curr_arg = 1 + *argv)) {

		/* Scan current argument until a flag indicates that the rest
		of the argument isn't flags (curr_arg = NULL), or until
		the end of the argument is reached (if it is all flags). */

		while (curr_arg != NULL && (c = *curr_arg++)) {
			switch (c) {

			default:
				fprintf(stderr, "%s: unknown flag -%c\n", Progname, c);
				USAGE();
				exit(-1);
			}
		}
	}

	/* Process the named file, or stdin if no file given.
	The name '-' also specifies stdin. */

	infile = stdin;
	if (argc && **argv != '-') {
		if ((infile = fopen(*argv, "r")) == NULL) {
			fprintf(stderr, "%s: can't open '%s'\n", Progname, *argv);
			exit(-1);
		}
	}

	doit(infile);

	exit(0);
}

#define DLASIZE 512
#define SIZE (DLASIZE + 2)      /* borders */
#define MAXLEVEL 20             /* should be lg(DLASIZE) */

char Dla[SIZE][SIZE];

#define ON 1
#define OFF 0

int Level_count[MAXLEVEL];

void
doit(FILE *infile)
{
	int i, j;
	char buf[100];

	/* Get the array. */

	for (i = 0; i < SIZE; i++) {
		for (j = 0; j < SIZE; j++) {
			Dla[i][j] = OFF;
		}
	}
	while (fgets(buf, 100, infile)) {
		sscanf(buf, "%d %d", &j, &i);
		Dla[i][j] = ON;
	}

	/* Set up the level counters. */

	for (i = 0; i < MAXLEVEL; i++) {
		Level_count[i] = 0;
	}

	/* Now start the recursion by asking the question:
	Is there something in this box? */

	Level_count[0] = box(1, 1, DLASIZE, 0);

	/* Output the counts. */

	for (i = 0; i < MAXLEVEL; i++) {
		if (Level_count[i] == 0) {
			break;
		}
		printf("%f %f\n", -log(DLASIZE >> i), log(Level_count[i]));
	}
}

/* Is there anything in the box of size size at (i, j)? */

int
box(int i, int j, int size, int level)
{
	int old;

	if (size == 1) {
		return (Dla[i][j] == ON);
	}

	/* Split the problem up into four smaller boxes. */

	level++;
	size >>= 1;
	old = Level_count[level];

	Level_count[level] += box(i, j, size, level);
	Level_count[level] += box(i, j + size, size, level);
	Level_count[level] += box(i + size, j, size, level);
	Level_count[level] += box(i + size, j + size, size, level);

	/* Return whether we saw something in one of the four boxes or not. */

	return (Level_count[level] != old);
}
